% Inference example for logistic regression using glm-ie demonstrating the 
% sparsity occuring in estimation but similar classification performance in
% inference where probabilities can readily be evaluated.
%
% We use the well known a9a dataset from the UCI machine learning repository
% with 32561 instances of dimension 123.
% B data matrix
% c binary labels
%
% (c) by Hannes Nickisch, MPI for Biological Cybernetics, 2010 August 03
clear all, close all

potS = @(s) potLaplace(s); % Laplace weight prior

% load and split into train and test set
load classify                               % load a part of the UCI a9a dataset
nte = 30000;                        % say how many test examples we wich to keep
Bte = B(1:nte,:); B = B(nte+1:end,:); cte = c(1:nte); c = c(nte+1:end);  % split
[q,n] = size(B);

% nearest neighbour classification
dp = sum(bsxfun(@minus, Bte, full(mean(B(c==+1,:)))).^2,2);  % dist. to pos. ctr
dm = sum(bsxfun(@minus, Bte, full(mean(B(c==-1,:)))).^2,2);  % dist. to neg. ctr
cc = 2*double(dp<dm)-1;
fprintf('nearest neighbor accuracy=%1.2f%%\n',100*sum(cte==cc)/numel(cte))

% classification
fprintf('Do regression using a ')
str = func2str(potS);
fprintf('%s',str(8:end))
m = 0; X = zeros(0,n); y = zeros(0,1); s2 = 1;         % no Gaussian part at all
tau  = [2*ones(n,1); 5*c.*ones(q,1)];
B = [eye(n);B];
pot = @(s) [potS(s(1:n)); potLogistic(s(n+(1:q)))];
fprintf(' weight prior.\n')

%% Inference
opts.innerOutput = 1;
opts.outerOutput = 1;
opts.outerNiter = 8;     % Number of outer loop iterations
opts.outerMVM = 170;     % Lanczos steps
opts.innerMVM = 100;     % CG steps
opts.innerPLS = 'plsTN'; % PLS algorithm

[uinf,ga,z,Q,T] = vbidl(X,y,s2,B,pot,tau,opts);
% compute standard deviation: sdinf = sqrt(s2*diag( Bte*Q*inv(T)*Q'*Bte' ));
% V = s2*inv(A), A = X'*X+B'*diag(1./ga)*B \approx Q*T*Q'
v = Bte*(Q/chol(T));
sdinf = s2*sqrt( sum(v.*v,2) );

%% Estimation
opt.nMVM = 250; opt.output = 1;
u0 = zeros(n,1);
uest = feval(opts.innerPLS,u0,X,y,B,opt,s2,'penFromPot',pot,tau/sqrt(s2));

fprintf('accuracies on test set\n')
acc_inference  = 100*sum(cte==sign(Bte*uinf))/numel(cte)
acc_estimation = 100*sum(cte==sign(Bte*uest))/numel(cte)

figure
plot(uinf,uest,'.'), xlabel('u_{inference}'), ylabel('u_{estimation}')
title('sparsity pattern: inference vs. estimation')