% Estimation example doing sparse regression for image deblurring using glm-ie.
%
% We do nonblind deconvolution to recover the image u that has been blurred by
% an optical device with known point spread function f.
% The measurements are y = conv2(u,f) + e, e~N(0,s2).
%
% (c) by Hannes Nickisch, MPI for Biological Cybernetics, 2010 July 22
clear all, close all

load deblur
ut = u; clear u, f = f(4:end,:); f = f/sum(f(:));    % image u and blur filter f
su = size(ut); nu = prod(su); sf = size(f);                              % sizes

% blur operator and noisy measurement
X  = cnv2mat(f, su, 'circ'); sy = get_sout(X);
s2 = 1e-5;                                                            % variance
y  = X*ut; y = y + sqrt(s2)*randn(size(y));                        % observation

% prior
B = fd2mat(su);                     % total variation = finite derivative matrix
tau = 15;                                                          % prior scale
pot = @(s) potT(s,2);

% optimisation parameters
opt.nMVM = 50; opt.output = 1;
u0 = zeros(nu,1);

%% Estimation: apply different PLS schemes
plsList = {'TN','CGBT','CG','LBFGS'};
pars = {u0,X,y,B,opt,s2,'penFromPot',pot,tau};
for i=1:length(plsList)
  fprintf('PLS optimisation using %s.\n',plsList{i})
  tic, [u{i},phi(i)] = feval(['pls',plsList{i}],pars{:}); t(i) = toc;
  d(i) = norm(u{i}-ut(:));
end

fprintf('objective function values\n')
for i=1:length(plsList), fprintf('%s %1.4e, ',plsList{i},phi(i)), end
fprintf('\b\b\n')

fprintf('running times\n')
for i=1:length(plsList), fprintf('%s %1.2fs, ',plsList{i},t(i)), end
fprintf('\b\b\n')

fprintf('accuracies in %%\n')
for i=1:length(plsList), fprintf('%s %1.2f%%, ',plsList{i},d(i)), end
fprintf('\b\b\n')

subplot(131), imagesc(reshape(ut,su)),   axis off, title('original')
subplot(132), imagesc(reshape(u{end},su)), axis off, title('deblurred')
subplot(133), imagesc(reshape(y,sy)), colormap(gray), axis off, title('blurry')