% Inference example doing sparse regression for image deblurring using glm-ie.
%
% We do nonblind deconvolution to recover the image u that has been blurred by
% an optical device with known point spread function f.
% The measurements are y = conv2(u,f) + e, e~N(0,s2).
%
% (c) by Hannes Nickisch, MPI for Biological Cybernetics, 2010 August 03
clear all, close all

load deblur
ut = u; clear u, f = f(4:end,:); f = f/sum(f(:));    % image u and blur filter f
su = size(ut); nu = prod(su); sf = size(f);                              % sizes

% blur operator and noisy measurement
X  = cnv2mat(f, su, 'circ'); sy = su;
s2 = 1e-5;                                                            % variance
y  = X*ut; y = y + sqrt(s2)*randn(size(y));                        % observation

% prior
B = fd2mat(su);                     % total variation = finite derivative matrix
tau = 15;                                                          % prior scale

pot = @(s) potLaplace(s);

%% Inference
opts.innerOutput = 1;
opts.outerOutput = 1;
opts.outerNiter = 8;     % Number of outer loop iterations
opts.outerMVM = 170;     % Lanczos steps
opts.innerMVM = 100;     % CG steps
opts.innerPLS = 'plsTN'; % PLS algorithm 
[uinf,ga,z,Q,T] = vbidl(X,y,s2,B,pot,tau,opts);
% compute standard deviation: sdinf = sqrt(s2*diag( Q*inv(T)*Q' ));
% V = inv(A), A = X'*X/s2 + B'*diag(1./ga)*B \approx Q*T*Q'
Q_T = Q/chol(T);
sdinf = sqrt( sum(Q_T.*Q_T,2) );
figure
  subplot(221), imagesc(reshape(ut,su)), axis off, title('original')
  subplot(223), imagesc(reshape(y,sy)), colormap(gray), axis off, title('blurry')
  subplot(222), imagesc(reshape(uinf,su)), axis off, title('inference mean')
  subplot(224), imagesc(reshape(sdinf,su)), axis off, title('inference std')


%% Estimation
opt.nMVM = 250; opt.output = 1;
u0 = zeros(nu,1);
uest = feval(opts.innerPLS,u0,X,y,B,opt,s2,'penFromPot',pot,tau);

% accuracy
err_estimation = norm(uest-ut(:))
err_inference  = norm(uinf-ut(:))

figure
  subplot(221), imagesc(reshape(ut,su)), axis off, title('original')
  subplot(223), imagesc(reshape(y,sy)), colormap(gray), axis off, title('blurry')
  subplot(222), imagesc(reshape(uinf,su)), axis off, title('inference deblurred')
  subplot(224), imagesc(reshape(uest,su)), axis off, title('estimation deblurred')