% [p,dp,d2p] = penFromPot(s,pot,tau,z,theta) - Potential derived penalty
% 
% pen(s) = tau*b*(r-s) - log( pot(tau*r) ) where r = sign(s)*sqrt(s^2 + z).
%
% Here, theta are additional parameters for the potential function pot
% which is invoked by the call feval(pot, tau.*r, theta{:}).
%
% For z->0, we have r->s and hence pen(s)->-log( pot(tau*s) ) => MAP estimation.
%
%   See also PENFUNCTIONS.M.
% (c) by Hannes Nickisch, MPI for Biological Cybernetics, 2010 August 05

function [p,dp,d2p] = penFromPot(s,pot,tau,z,varargin)

if nargin<4, z = 1e-6; end             % default value for smoothing parameter z

sign_s = 2*(s>=0)-1;                                % strict sign mapping 0 to 1 
r = sign_s.*sqrt(s.*s + z);                         % signed z-smoothed variable
[lp,dlp,d2lp,b] = cols( feval(pot,tau.*r,varargin{:}) );   % evaluate potentials
p   = tau.*b.*(r-s) - lp;                                              % penalty
s_r = (abs(s)+eps)./(abs(r)+eps);         % stabilised s./r to cover z=0 and s=0
dp  = tau.*((b-dlp).*s_r - b);                                % first derivative
r_e = r + sign_s*eps;
z_r3  = z./(r_e.*r_e.*r_e);            % stabilised z./r.^3 to cover z=0 and s=0
d2p = tau.*((b-dlp).*z_r3 - tau.*d2lp.*s_r.*s_r);            % second derivative

id = abs(s./sqrt(z))>1e10;                  % correct asymptotics of s -> +/-Inf
p(id) = -lp(id);  dp(id) = -dlp(id);  d2p(id) = -d2lp(id);

% split a matrix into its columns
function varargout = cols(A)
  ncol = size(A,2); varargout = cell(1,nargout);
  for n=1:nargout, if n>ncol, varargout{n}=[]; else varargout{n}=A(:,n); end,end